package se.su.dsv.scipro.project.panels;

import org.apache.wicket.ajax.AjaxRequestTarget;
import org.apache.wicket.ajax.markup.html.AjaxLink;
import org.apache.wicket.markup.html.WebMarkupContainer;
import org.apache.wicket.markup.html.panel.Panel;
import org.apache.wicket.model.IModel;
import org.apache.wicket.model.ResourceModel;
import se.su.dsv.scipro.components.AuthorAutoComplete;
import se.su.dsv.scipro.components.InfoPanel;
import se.su.dsv.scipro.data.DetachableServiceModel;
import se.su.dsv.scipro.profile.UserLinkPanel;
import se.su.dsv.scipro.system.User;
import se.su.dsv.scipro.system.UserService;

import javax.inject.Inject;

public class PartnerAdditionPanel extends Panel {

    public static final String CO_AUTHOR_LABEL = "coAuthorLabel";

    @Inject
    private UserService userService;

    private IModel<User> coAuthor = new DetachableServiceModel<>(userService);
    private AuthorAutoComplete coAuthorField;
    private WebMarkupContainer coAuthorContainer;
    private AjaxLink<Void> removeLink;

    public PartnerAdditionPanel(String id) {
        super(id);
        coAuthorContainer = new WebMarkupContainer("container");
        coAuthorContainer.setOutputMarkupId(true);
        add(coAuthorContainer);
        addAutoCompleteField();
        addCurrentCoAuthorLabel();
        addRemoveAuthorIcon();
        addInfo();
    }

    private void addInfo() {
        coAuthorContainer.add(new InfoPanel("partnerInfo", new ResourceModel("partnerInfo")));
    }

    private void addAutoCompleteField() {
        coAuthorField = new AuthorAutoComplete("inputField", coAuthor) {

            @Override
            protected void action(AjaxRequestTarget pTarget, User newSelection) {
                setVisible(false);
                removeLink.setVisible(true);
                pTarget.add(coAuthorContainer);
            }
        };
        coAuthorField.setVisible(!coAuthorExists());
        coAuthorContainer.add(coAuthorField);
    }

    private void addCurrentCoAuthorLabel() {
        coAuthorContainer.add(new UserLinkPanel(CO_AUTHOR_LABEL, coAuthor));
    }

    private void addRemoveAuthorIcon() {
        removeLink = new AjaxLink<Void>("link") {

            @Override
            public void onClick(AjaxRequestTarget target) {
                removeCoAuthor();
                target.add(coAuthorContainer);
            }

        };
        removeLink.setVisible(coAuthorExists());
        coAuthorContainer.add(removeLink);
    }

    public final void removeCoAuthor() {
        if (coAuthorExists()) {
            coAuthor.setObject(null);
            removeLink.setVisible(false);
            coAuthorField.setVisible(true);
        }
    }

    public User getCoAuthor() {
        return coAuthor.getObject();
    }

    private boolean coAuthorExists() {
        return coAuthor.getObject() != null;
    }
}
